@@ -25,8 +25,10 @@ class ResultStream(object):
         self.orig_stream = orig_stream
         self.logfile = None
         self.logger = None  # is set in the TestRunner class
-        self.onlylogtologfile = True
+        self.debug = False
         self.ansi_escape = re.compile(r'\x1b[^m]*m')
+        self.nocolfmt = Formatter(pre=nocolors, lenstrip=None, contline=None)
+        self.colfmt = Formatter()
 
     def flush(self):
         self.orig_stream.flush()
@@ -38,40 +40,52 @@ class ResultStream(object):
         if self.logfile:
             self.logfile.write(self.ansi_escape.sub('', text))
 
-    def open_file(self, fname, warnstream):
-        self.logfile = open(fname, "w")
+    def before_test(self, logfilename, warnstream):
+        if logfilename:
+            self.logfile = open(logfilename, "w")
         if self.logger is None:
             return
-        nocolfmt = Formatter(pre=nocolors, lenstrip=None, contline=None)
-        if self.onlylogtologfile:
-            hdl = logging.StreamHandler(self.logfile)
-            fmt = nocolfmt
-        else:
+
+        if self.debug:
+            # write to orig_stream and to logfile (if specified)
             hdl = logging.StreamHandler(self)
-            fmt = Formatter()
-        hdl.setFormatter(fmt)
+            hdl.setFormatter(self.colfmt)
+        elif self.logfile:
+            # log messages are only written to the logfiles and to
+            # orig-stream. The ansi escape sequences in the redgreentest
+            # output get removed from the logfile (`self.ansi_escape`)
+            hdl = logging.StreamHandler(self.logfile)
+            hdl.setFormatter(self.nocolfmt)
+        else:
+            # no logfile output + no logger output to orig_stream. exception:
+            # the warnings/errors summary at the end of each test is written
+            # to orig_stream
+            hdl = logging.NullHandler()
         self.logger.addHandler(hdl)
         hdl = logging.StreamHandler(warnstream)
         hdl.setLevel(logging.WARNING)
-        hdl.setFormatter(nocolfmt)
+        hdl.setFormatter(self.nocolfmt)
         self.logger.addHandler(hdl)
 
-    def close_file(self):
-        if not self.logfile:
+    def after_test(self):
+        if self.logfile:
+            self.logfile.close()
+            self.logfile = None
+
+        if self.logger is None:
             return
 
-        self.logfile.close()
-        self.logfile = None
-
-        if self.logger is not None:
-            self.logger.removeHandler(self.logger.handlers[0])
-            self.logger.removeHandler(self.logger.handlers[0])
+        # handler for warnstream
+        self.logger.removeHandler(self.logger.handlers[-1])
+        # stream or nullhandler
+        self.logger.removeHandler(self.logger.handlers[-1])
 
 
 class ResultHandler(RedGreenTextTestResult):
 
     ipdb = False
     width = 80
+    logdirectory = None
 
     def startTest(self, test):
         # increments the number of run tests counter
@@ -80,9 +94,13 @@ class ResultHandler(RedGreenTextTestResult):
         tname = '%s.%s' % (test.__class__.__name__, test._testMethodName)
         module = test.__class__.__module__
         desc = ' %s in %s ' % (tname, module)
-        logfile = os.path.join('log', '%s.log' % (tname.replace('.', '_')))
+
         test.warningserrors = StringIO()
-        self.stream.open_file(logfile, test.warningserrors)
+        logfile = None
+        if self.logdirectory:
+            logfile = os.path.join(self.logdirectory,
+                                   '%s.log' % (tname.replace('.', '_')))
+        self.stream.before_test(logfile, test.warningserrors)
         self.stream.writeln(desc.center(self.width, '-'))
         self.stream.flush()
 
@@ -105,7 +123,7 @@ class ResultHandler(RedGreenTextTestResult):
             self.stream.writeln('\n'.join(s), WARNING)
 
         self.stream.writeln(self.separator2)
-        self.stream.close_file()
+        self.stream.after_test()
         self.stream.writeln('')
 
     def addError(self, test, err):
@@ -167,14 +185,37 @@ class ResultHandler(RedGreenTextTestResult):
 class FalafelTestRunner(RedGreenTextTestRunner):
 
     def __init__(self, *args, **kwargs):
-        kwargs["stream"] = ResultStream(sys.stderr)
+        """
+        Parameters
+        ----------
+        logdirectory : str, optional
+            If set, specifies the directory where all logfiles are stored.
+            Otherwise, test output is only written to stdout.
+        debug : bool
+            Write debug log messages to stdout. Note that debug messages are
+            per default written to the logfiles given that `logdirectory` was
+            specified. Only has an effect if a python logger was set.
+        logger : python logger instance, str, optional
+        """
+        # TODO add more kwargs to control the behavior of the ResultHandler
+        # class and of the ResultStream class
+
+        if 'stream' not in kwargs:
+            kwargs["stream"] = ResultStream(sys.stderr)
         kwargs["resultclass"] = ResultHandler
+        ResultHandler.logdirectory = kwargs.pop('logdirectory')
+        if ResultHandler.logdirectory and not os.path.exists(
+                ResultHandler.logdirectory):
+            print("creating logdirectory: '%s'" % ResultHandler.logdirectory)
+            os.makedirs(ResultHandler.logdirectory)
 
         logger = kwargs.pop('logger', None)
-        if logger is not None:
+        if logger is not None and isinstance(kwargs['stream'], ResultStream):
+            if isinstance(logger, str):
+                logger = logging.getLogger(logger)
+                logger.setLevel(logging.DEBUG)
+                logger.propagate = False
             kwargs['stream'].logger = logger
-            if kwargs.pop('debug', False):
-                # log to logfile as well to stdout
-                kwargs['stream'].onlylogtologfile = False
+            kwargs['stream'].debug = kwargs.pop('debug', False)
 
         super(FalafelTestRunner, self).__init__(*args, **kwargs)
